@@ -152,7 +152,7 @@ class Actions(JailThread, Mapping):
 					if name in self._actions:
 						self._actions[name].reload(**(initOpts if initOpts else {}))
 				# remove obsolete actions (untouched by reload process):
-				delacts = OrderedDict((name, action) for name, action in self._actions.items()
+				delacts = OrderedDict((name, action) for name, action in iter(self._actions.items())
 					if name not in self._reload_actions)
 				if len(delacts):
 					# unban all tickets using remove action only:
