@@ -354,7 +354,7 @@ class BanManager:
 						del self.__banList[fid]
 				else:
 					# create new dictionary without items to be deleted:
-					self.__banList = dict((fid,ticket) for fid,ticket in self.__banList.items() \
+					self.__banList = dict((fid,ticket) for fid,ticket in iter(self.__banList.items()) \
 						if fid not in unBanList)
 						
 			# return list of tickets:
