@@ -64,7 +64,7 @@ if sys.version_info >= (3,):
 else:
 	def _normalize(x):
 		if isinstance(x, dict):
-			return dict((_normalize(k), _normalize(v)) for k, v in x.items())
+			return dict((_normalize(k), _normalize(v)) for k, v in iter(x.items()))
 		elif isinstance(x, (list, set)):
 			return [_normalize(element) for element in x]
 		elif isinstance(x, str):
