@@ -61,7 +61,7 @@ class Ticket(object):
 					self._data[k] = v
 		if ticket:
 			# ticket available - copy whole information from ticket:
-			self.__dict__.update(i for i in ticket.__dict__.items() if i[0] in self.__dict__)
+			self.__dict__.update(i for i in iter(ticket.__dict__.items()) if i[0] in self.__dict__)
 
 	def __str__(self):
 		return "%s: ip=%s time=%s #attempts=%d matches=%r" % \
@@ -171,7 +171,7 @@ class Ticket(object):
 		elif len(args) == 2:
 			self._data.update((args,))
 		elif len(args) > 2:
-			self._data.update((k,v) for k,v in zip(*[iter(args)]*2))
+			self._data.update((k,v) for k,v in list(zip(*[iter(args)]*2)))
 		if len(argv):
 			self._data.update(argv)
 		# filter (delete) None values:
